@@ -280,7 +280,7 @@ dbcontext::init_thread(const void *stack_bottom, volatile int& shutdown_flag)
     DBG_THR(fprintf(stderr,
       "thread_stack = %p sizeof(THD)=%zu sizeof(mtx)=%zu "
       "O: %zu %zu %zu %zu %zu %zu %zu\n",
-      thd->thread_stack, sizeof(THD), sizeof(LOCK_thread_count),
+      thd->thread_stack, sizeof(THD), sizeof(mysql_mutex_t),
       DENA_THR_OFFSETOF(mdl_context),
       DENA_THR_OFFSETOF(net),
       DENA_THR_OFFSETOF(LOCK_thd_data),
@@ -297,9 +297,9 @@ dbcontext::init_thread(const void *stack_bottom, volatile int& shutdown_flag)
       #else
       thd->options |= OPTION_BIN_LOG;
       #endif
-      safeFree(thd->db);
-      thd->db = 0;
-      thd->db = my_strdup("handlersocket", MYF(0));
+      safeFree((char*) thd->db.str);
+      thd->db.str= my_strdup("handlersocket", MYF(0));
+      thd->db.length= sizeof("handlersocket")-1;
     }
     thd->variables.option_bits |= OPTION_TABLE_LOCK;
     my_pthread_setspecific_ptr(THR_THD, thd);
@@ -307,7 +307,7 @@ dbcontext::init_thread(const void *stack_bottom, volatile int& shutdown_flag)
   }
   {
     thd->thread_id = next_thread_id();
-    add_to_active_threads(thd);
+    server_threads.insert(thd);
   }
 
   DBG_THR(fprintf(stderr, "HNDSOCK init thread wsts\n"));
@@ -341,10 +341,8 @@ dbcontext::term_thread()
   close_tables_if();
   my_pthread_setspecific_ptr(THR_THD, 0);
   {
-    pthread_mutex_lock(&LOCK_thread_count);
     delete thd;
     thd = 0;
-    pthread_mutex_unlock(&LOCK_thread_count);
     my_thread_end();
   }
 }
@@ -473,9 +471,7 @@ dbcontext::close_tables_if()
   unlock_tables_if();
   DENA_VERBOSE(100, fprintf(stderr, "HNDSOCK close tables\n"));
   close_thread_tables(thd);
-  #if MYSQL_VERSION_ID >= 50505
-  thd->mdl_context.release_transactional_locks();
-  #endif
+  thd->mdl_context.release_transactional_locks(thd);
   if (!table_vec.empty()) {
     statistic_increment(close_tables_count, &LOCK_status);
     table_vec.clear();
@@ -1005,8 +1001,9 @@ dbcontext::cmd_open(dbcallback_i& cb, const cmd_open_args& arg)
     bool refresh = true;
     const thr_lock_type lock_type = for_write_flag ? TL_WRITE : TL_READ;
     #if MYSQL_VERSION_ID >= 50505
-    tables.init_one_table(arg.dbn, strlen(arg.dbn), arg.tbl, strlen(arg.tbl),
-      arg.tbl, lock_type);
+    LEX_CSTRING db_name=  { arg.dbn, strlen(arg.dbn) };
+    LEX_CSTRING tbl_name= { arg.tbl, strlen(arg.tbl) };
+    tables.init_one_table(&db_name, &tbl_name, 0, lock_type);
     tables.mdl_request.init(MDL_key::TABLE, arg.dbn, arg.tbl,
       for_write_flag ? MDL_SHARED_WRITE : MDL_SHARED_READ, MDL_TRANSACTION);
     Open_table_context ot_act(thd, 0);
