@@ -152,6 +152,7 @@ public:
     global chain of TABLE_LIST objects created for the whole query.
   */
   With_element_head *head;
+
   /*
     Optional list of column names to name the columns of the table introduced
     by this with element. It is used in the case when the names are not
@@ -253,7 +254,7 @@ public:
 
   TABLE_LIST *find_first_sq_rec_ref_in_select(st_select_lex *sel);
 
-  bool set_unparsed_spec(THD *thd, char *spec_start, char *spec_end,
+  bool set_unparsed_spec(THD *thd, const char *spec_start, const char *spec_end,
                          my_ptrdiff_t spec_offset);
 
   st_select_lex_unit *clone_parsed_spec(LEX *old_lex, TABLE_LIST *with_table);
@@ -356,8 +357,7 @@ private:
   */
   With_clause *next_with_clause;
   /* Set to true if dependencies between with elements have been checked */
-  bool dependencies_are_checked; 
-
+  bool dependencies_are_checked;
   /* 
     The bitmap of all recursive with elements whose specifications
     are not complied with restrictions imposed by the SQL standards
@@ -381,9 +381,8 @@ public:
   bool with_recursive;
 
   With_clause(bool recursive_fl, With_clause *emb_with_clause)
-    : owner(NULL),
-      embedding_with_clause(emb_with_clause), next_with_clause(NULL),
-      dependencies_are_checked(false),  unrestricted(0),
+    : owner(NULL), embedding_with_clause(emb_with_clause),
+      next_with_clause(NULL), dependencies_are_checked(false), unrestricted(0),
       with_prepared_anchor(0), cleaned(0), stabilized(0),
       with_recursive(recursive_fl)
   { }
@@ -391,28 +390,18 @@ public:
   bool add_with_element(With_element *elem);
 
   /* Add this with clause to the list of with clauses used in the statement */
-  void add_to_list(With_clause **ptr, With_clause ** &last_next)
+  void add_to_list(With_clause ** &last_next)
   {
-    if (embedding_with_clause)
-    {
-      /* 
-        An embedded with clause is always placed before the embedding one
-        in the list of with clauses used in the query.
-      */
-      while (*ptr != embedding_with_clause)
-        ptr= &(*ptr)->next_with_clause;
-      *ptr= this;
-      next_with_clause= embedding_with_clause;
-    }
-    else
-    {
-      *last_next= this;
-      last_next= &this->next_with_clause;
-    }
+    *last_next= this;
+    last_next= &this->next_with_clause;
   }
 
+  st_select_lex_unit *get_owner() { return owner; }
+
   void set_owner(st_select_lex_unit *unit) { owner= unit; }
 
+  void attach_to(st_select_lex *select_lex);
+
   With_clause *pop() { return embedding_with_clause; }
       
   bool check_dependencies();
@@ -447,7 +436,6 @@ bool With_element::is_unrestricted()
 }
 
 inline
-
 bool With_element::is_with_prepared_anchor() 
 {
   return owner->with_prepared_anchor & get_elem_map();
@@ -529,11 +517,14 @@ void With_element::prepare_for_next_iteration()
 
 
 inline
-void  st_select_lex_unit::set_with_clause(With_clause *with_cl)
-{ 
-    with_clause= with_cl;
-    if (with_clause)
-      with_clause->set_owner(this);
+void With_clause::attach_to(st_select_lex *select_lex)
+{
+  for (With_element *with_elem= with_list.first;
+       with_elem;
+       with_elem= with_elem->next)
+  {
+    select_lex->register_unit(with_elem->spec, NULL);
+  }
 }
 
 
