@@ -239,7 +239,7 @@ int JSONDISC::GetColumns(PGLOBAL g, PCSZ db, PCSZ dsn, PTOS topt)
       (tdp->Version == 2) ? "Mongo2Interface" : "Mongo3Interface");
     tdp->Pretty = 0;
 #else   // !MONGO_SUPPORT
-    snprintf(g->Message, sizeof(g->Message), MSG(NO_FEAT_SUPPORT), "MONGO");
+    sprintf(g->Message, MSG(NO_FEAT_SUPPORT), "MONGO");
     return 0;
 #endif  // !MONGO_SUPPORT
   } // endif Uri
@@ -249,7 +249,7 @@ int JSONDISC::GetColumns(PGLOBAL g, PCSZ db, PCSZ dsn, PTOS topt)
 #if defined(ZIP_SUPPORT)
       tjsp = new(g) TDBJSON(tdp, new(g) UNZFAM(tdp));
 #else   // !ZIP_SUPPORT
-      snprintf(g->Message, sizeof(g->Message), MSG(NO_FEAT_SUPPORT), "ZIP");
+      sprintf(g->Message, MSG(NO_FEAT_SUPPORT), "ZIP");
       return 0;
 #endif  // !ZIP_SUPPORT
     } else
@@ -260,14 +260,14 @@ int JSONDISC::GetColumns(PGLOBAL g, PCSZ db, PCSZ dsn, PTOS topt)
 
     jsp = (tjsp->GetDoc()) ? tjsp->GetDoc()->GetArrayValue(0) : NULL;
   } else {
-		if (!((tdp->Lrecl = GetIntegerTableOption(g, topt, "Lrecl", 0)))) {
-			if (!mgo && !tdp->Uri) {
-				snprintf(g->Message, sizeof(g->Message), "LRECL must be specified for pretty=%d", tdp->Pretty);
-				return 0;
+    if (!(tdp->Lrecl = GetIntegerTableOption(g, topt, "Lrecl", 0)))
+    {
+      if (!mgo && !tdp->Uri) {
+        sprintf(g->Message, "LRECL must be specified for pretty=%d", tdp->Pretty);
+        return 0;
       } else
         tdp->Lrecl = 8192;       // Should be enough
-
-		} // endif Lrecl
+    }
 
     tdp->Ending = GetIntegerTableOption(g, topt, "Ending", CRLF);
 
@@ -275,7 +275,7 @@ int JSONDISC::GetColumns(PGLOBAL g, PCSZ db, PCSZ dsn, PTOS topt)
 #if defined(ZIP_SUPPORT)
       tjnp = new(g)TDBJSN(tdp, new(g) UNZFAM(tdp));
 #else   // !ZIP_SUPPORT
-      snprintf(g->Message, sizeof(g->Message), MSG(NO_FEAT_SUPPORT), "ZIP");
+      sprintf(g->Message, MSG(NO_FEAT_SUPPORT), "ZIP");
       return NULL;
 #endif  // !ZIP_SUPPORT
     } else if (tdp->Uri) {
@@ -283,14 +283,14 @@ int JSONDISC::GetColumns(PGLOBAL g, PCSZ db, PCSZ dsn, PTOS topt)
 #if defined(CMGO_SUPPORT)
         tjnp = new(g) TDBJSN(tdp, new(g) CMGFAM(tdp));
 #else
-        snprintf(g->Message, sizeof(g->Message), "Mongo %s Driver not available", "C");
+        sprintf(g->Message, "Mongo %s Driver not available", "C");
         return 0;
 #endif
       } else if (tdp->Driver && toupper(*tdp->Driver) == 'J') {
 #if defined(JAVA_SUPPORT)
         tjnp = new(g) TDBJSN(tdp, new(g) JMGFAM(tdp));
 #else
-        snprintf(g->Message, sizeof(g->Message), "Mongo %s Driver not available", "Java");
+        sprintf(g->Message, "Mongo %s Driver not available", "Java");
         return 0;
 #endif
       } else {             // Driver not specified
@@ -299,7 +299,7 @@ int JSONDISC::GetColumns(PGLOBAL g, PCSZ db, PCSZ dsn, PTOS topt)
 #elif defined(JAVA_SUPPORT)
         tjnp = new(g) TDBJSN(tdp, new(g) JMGFAM(tdp));
 #else
-        snprintf(g->Message, sizeof(g->Message), MSG(NO_FEAT_SUPPORT), "MONGO");
+        sprintf(g->Message, MSG(NO_FEAT_SUPPORT), "MONGO");
         return 0;
 #endif
       } // endif Driver
@@ -513,7 +513,7 @@ bool JSONDISC::Find(PGLOBAL g, PJVAL jvp, PCSZ key, int j)
 
         return false;
       default:
-        snprintf(g->Message, sizeof(g->Message), "Logical error after %s", fmt);
+        sprintf(g->Message, "Logical error after %s", fmt);
         return true;
     } // endswitch Type
 
@@ -671,7 +671,7 @@ bool JSONDEF::DefineAM(PGLOBAL g, LPCSTR am, int poff)
       Wrapname = GetStringCatInfo(g, "Wrapper", "Mongo3Interface");
 #endif   // JAVA_SUPPORT
 #else   // !MONGO_SUPPORT
-    snprintf(g->Message, sizeof(g->Message), MSG(NO_FEAT_SUPPORT), "MONGO");
+    sprintf(g->Message, MSG(NO_FEAT_SUPPORT), "MONGO");
     return true;
 #endif  // !MONGO_SUPPORT
   } // endif Uri
@@ -706,14 +706,14 @@ PTDB JSONDEF::GetTable(PGLOBAL g, MODE m)
 #if defined(CMGO_SUPPORT)
       txfp = new(g) CMGFAM(this);
 #else
-      snprintf(g->Message, sizeof(g->Message), "Mongo %s Driver not available", "C");
+      sprintf(g->Message, "Mongo %s Driver not available", "C");
       return NULL;
 #endif
       } else if (Driver && toupper(*Driver) == 'J') {
 #if defined(JAVA_SUPPORT)
         txfp = new(g) JMGFAM(this);
 #else
-        snprintf(g->Message, sizeof(g->Message), "Mongo %s Driver not available", "Java");
+        sprintf(g->Message, "Mongo %s Driver not available", "Java");
         return NULL;
 #endif
       } else {             // Driver not specified
@@ -722,7 +722,7 @@ PTDB JSONDEF::GetTable(PGLOBAL g, MODE m)
 #elif defined(JAVA_SUPPORT)
         txfp = new(g) JMGFAM(this);
 #else   // !MONGO_SUPPORT
-        snprintf(g->Message, sizeof(g->Message), MSG(NO_FEAT_SUPPORT), "MONGO");
+        sprintf(g->Message, MSG(NO_FEAT_SUPPORT), "MONGO");
         return NULL;
 #endif  // !MONGO_SUPPORT
       } // endif Driver
@@ -739,7 +739,7 @@ PTDB JSONDEF::GetTable(PGLOBAL g, MODE m)
         return NULL;
       } // endif's m
 #else   // !ZIP_SUPPORT
-      snprintf(g->Message, sizeof(g->Message), MSG(NO_FEAT_SUPPORT), "ZIP");
+      sprintf(g->Message, MSG(NO_FEAT_SUPPORT), "ZIP");
       return NULL;
 #endif  // !ZIP_SUPPORT
     } else if (Compressed) {
@@ -749,7 +749,7 @@ PTDB JSONDEF::GetTable(PGLOBAL g, MODE m)
       else
         txfp = new(g) ZLBFAM(this);
 #else   // !GZ_SUPPORT
-      snprintf(g->Message, sizeof(g->Message), MSG(NO_FEAT_SUPPORT), "GZ");
+      sprintf(g->Message, MSG(NO_FEAT_SUPPORT), "GZ");
       return NULL;
 #endif  // !GZ_SUPPORT
     } else if (map)
@@ -792,7 +792,7 @@ PTDB JSONDEF::GetTable(PGLOBAL g, MODE m)
         return NULL;
       } // endif's m
 #else   // !ZIP_SUPPORT
-      snprintf(g->Message, sizeof(g->Message), MSG(NO_FEAT_SUPPORT), "ZIP");
+      sprintf(g->Message, MSG(NO_FEAT_SUPPORT), "ZIP");
       return NULL;
 #endif  // !ZIP_SUPPORT
     } else
@@ -980,7 +980,7 @@ PJSON TDBJSN::FindRow(PGLOBAL g)
     } else {
       if (bp || *objpath == '[') {
         if (objpath[strlen(objpath) - 1] != ']') {
-          snprintf(g->Message, sizeof(g->Message), "Invalid Table path %s", Objname);
+          sprintf(g->Message, "Invalid Table path %s", Objname);
           return NULL;
         } else if (!bp)
           objpath++;
@@ -1031,7 +1031,7 @@ bool TDBJSN::OpenDB(PGLOBAL g)
         case MODE_ARRAY:  Row = new(g) JARRAY;  break;
         case MODE_VALUE:  Row = new(g) JVALUE;  break;
         default:
-          snprintf(g->Message, sizeof(g->Message), "Invalid Jmode %d", Jmode);
+          sprintf(g->Message, "Invalid Jmode %d", Jmode);
           return true;
         } // endswitch Jmode
 
@@ -1210,7 +1210,7 @@ bool TDBJSN::MakeTopTree(PGLOBAL g, PJSON jsp)
           if (bp || *objpath == '[') {
             // Old style
             if (objpath[strlen(objpath) - 1] != ']') {
-              snprintf(g->Message, sizeof(g->Message), "Invalid Table path %s", Objname);
+              sprintf(g->Message, "Invalid Table path %s", Objname);
               return true;
             } else if (!bp)
               objpath++;
@@ -1259,7 +1259,7 @@ bool TDBJSN::PrepareWriting(PGLOBAL g)
 
     if ((signed)strlen(s) > Lrecl) {
       strncpy(To_Line, s, Lrecl);
-      snprintf(g->Message, sizeof(g->Message), "Line truncated (lrecl=%d)", Lrecl);
+      sprintf(g->Message, "Line truncated (lrecl=%d)", Lrecl);
       return PushWarning(g, this);
     } else
       strcpy(To_Line, s);
@@ -1384,7 +1384,7 @@ bool JSONCOL::SetArrayOptions(PGLOBAL g, char *p, int i, PSZ nm)
       p[--n] = 0;
     } else if (!IsNum(p)) {
       // Wrong array specification
-      snprintf(g->Message, sizeof(g->Message), "Invalid array specification %s for %s", p, Name);
+      sprintf(g->Message, "Invalid array specification %s for %s", p, Name);
       return true;
     } // endif p
 
@@ -1443,7 +1443,7 @@ bool JSONCOL::SetArrayOptions(PGLOBAL g, char *p, int i, PSZ nm)
 
         break;
       default:
-        snprintf(g->Message, sizeof(g->Message),
+        sprintf(g->Message,
           "Invalid function specification %c for %s", *p, Name);
         return true;
     } // endswitch *p
@@ -1459,7 +1459,7 @@ bool JSONCOL::SetArrayOptions(PGLOBAL g, char *p, int i, PSZ nm)
     } // endif n
 
   } else {
-    snprintf(g->Message, sizeof(g->Message), "Wrong array specification for %s", Name);
+    sprintf(g->Message, "Wrong array specification for %s", Name);
     return true;
   } // endif's
 
@@ -1528,7 +1528,7 @@ bool JSONCOL::ParseJpath(PGLOBAL g)
         goto fin;
       } // endif Name
 
-    snprintf(g->Message, sizeof(g->Message), "Cannot parse updated column %s", Name);
+    sprintf(g->Message, "Cannot parse updated column %s", Name);
     return true;
   } // endif To_Orig
 
@@ -1613,6 +1613,7 @@ PSZ JSONCOL::GetJpath(PGLOBAL g, bool proj)
     } // endif
 
     for (p1 = p2 = mgopath; *p1; p1++)
+    {
       if (i) {                 // Inside []
         if (isdigit(*p1)) {
           if (!proj)
@@ -1651,11 +1652,12 @@ PSZ JSONCOL::GetJpath(PGLOBAL g, bool proj)
             Sgfy = true;
             break;
           } // endif p2
-
+          /* falls through */
         default:
           *p2++ = *p1;
           break;
       } // endswitch p1;
+    }
 
       if (*(p2 - 1) == '.')
         p2--;
@@ -1742,7 +1744,7 @@ PJVAL JSONCOL::GetRowValue(PGLOBAL g, PJSON row, int i)
         val = (PJVAL)row;
         break;
       default:
-        snprintf(g->Message, sizeof(g->Message), "Invalid row JSON type %d", row->GetType());
+        sprintf(g->Message, "Invalid row JSON type %d", row->GetType());
         val = NULL;
     } // endswitch Type
 
@@ -1804,7 +1806,7 @@ void JSONCOL::SetJsonValue(PGLOBAL g, PVAL vp, PJVAL jvp)
 
           break;
         default:
-					snprintf(g->Message, sizeof(g->Message), "Unsupported column type %d\n", vp->GetType());
+					sprintf(g->Message, "Unsupported column type %d\n", vp->GetType());
 					throw 888;
 				} // endswitch Type
 
@@ -1905,7 +1907,7 @@ PVAL JSONCOL::GetColumnValue(PGLOBAL g, PJSON row, int i)
         val = (PJVAL)row;
         break;
       default:
-        snprintf(g->Message, sizeof(g->Message), "Invalid row JSON type %d", row->GetType());
+        sprintf(g->Message, "Invalid row JSON type %d", row->GetType());
         val = NULL;
       } // endswitch Type
 
@@ -2100,7 +2102,7 @@ PJSON JSONCOL::GetRow(PGLOBAL g)
         val = (PJVAL)row;
         break;
       default:
-        snprintf(g->Message, sizeof(g->Message), "Invalid row JSON type %d", row->GetType());
+        sprintf(g->Message, "Invalid row JSON type %d", row->GetType());
         val = NULL;
       } // endswitch Type
 
@@ -2227,7 +2229,7 @@ void JSONCOL::WriteColumn(PGLOBAL g)
 
       break;
     default:                  // ??????????
-      snprintf(g->Message, sizeof(g->Message), "Invalid column type %d", Buf_Type);
+      sprintf(g->Message, "Invalid column type %d", Buf_Type);
     } // endswitch Type
 
 } // end of WriteColumn
@@ -2373,7 +2375,7 @@ int TDBJSON::MakeDocument(PGLOBAL g)
         val = objp->GetKeyValue(key);
 
         if (!val || !(jsp = val->GetJson())) {
-          snprintf(g->Message, sizeof(g->Message), "Cannot find object key %s", key);
+          sprintf(g->Message, "Cannot find object key %s", key);
           return RC_FX;
         } // endif val
 
@@ -2381,7 +2383,7 @@ int TDBJSON::MakeDocument(PGLOBAL g)
         if (*p == '[') {
           // Old style
           if (p[strlen(p) - 1] != ']') {
-            snprintf(g->Message, sizeof(g->Message), "Invalid Table path near %s", p);
+            sprintf(g->Message, "Invalid Table path near %s", p);
             return RC_FX;
           } else
             p++;
@@ -2399,7 +2401,7 @@ int TDBJSON::MakeDocument(PGLOBAL g)
         val = arp->GetArrayValue(i);
 
         if (!val) {
-          snprintf(g->Message, sizeof(g->Message), "Cannot find array value %d", i);
+          sprintf(g->Message, "Cannot find array value %d", i);
           return RC_FX;
         } // endif val
 
@@ -2444,16 +2446,15 @@ int TDBJSON::Cardinality(PGLOBAL g)
 {
   if (!g)
     return (Xcol || Multiple) ? 0 : 1;
-	else if (Cardinal < 0) {
-		if (!Multiple) {
-			if (MakeDocument(g) == RC_OK)
-				Cardinal = Doc->size();
-
-		} else
-			return 10;
-
-	} // endif Cardinal
+  else if (Cardinal < 0)
+  {
+    if (!Multiple) {
+      if (MakeDocument(g) == RC_OK)
+        Cardinal = Doc->size();
 
+    } else
+      return 10;
+  }
   return Cardinal;
 } // end of Cardinality
 
@@ -2564,7 +2565,7 @@ bool TDBJSON::OpenDB(PGLOBAL g)
       case MODE_ARRAY:  Row = new(g) JARRAY;  break;
       case MODE_VALUE:  Row = new(g) JVALUE;  break;
       default:
-        snprintf(g->Message, sizeof(g->Message), "Invalid Jmode %d", Jmode);
+        sprintf(g->Message, "Invalid Jmode %d", Jmode);
         return true;
       } // endswitch Jmode
 
@@ -2648,7 +2649,7 @@ int TDBJSON::DeleteDB(PGLOBAL g, int irc)
   if (irc == RC_OK) {
     // Deleted current row
     if (Doc->DeleteValue(Fpos)) {
-      snprintf(g->Message, sizeof(g->Message), "Value %d does not exist", Fpos + 1);
+      sprintf(g->Message, "Value %d does not exist", Fpos + 1);
       return RC_FX;
       } // endif Delete
 
