@@ -13,7 +13,7 @@ FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
-51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -135,7 +135,7 @@ fts_eval_sql(
 /** Construct the name of an internal FTS table for the given table.
 @param[in]	fts_table	metadata on fulltext-indexed table
 @param[out]	table_name	a name up to MAX_FULL_NAME_LEN
-@param[in]	dict_locked	whether dict_sys->mutex is being held */
+@param[in]	dict_locked	whether dict_sys.mutex is being held */
 void fts_get_table_name(const fts_table_t* fts_table, char* table_name,
 			bool dict_locked = false)
 	MY_ATTRIBUTE((nonnull));
@@ -239,7 +239,7 @@ Do a binary search for a doc id in the array
 int
 fts_bsearch(
 /*========*/
-	fts_update_t*	array,		/*!< in: array to sort */
+	doc_id_t*	array,		/*!< in: array to sort */
 	int		lower,		/*!< in: lower bound of array*/
 	int		upper,		/*!< in: upper bound of array*/
 	doc_id_t	doc_id)		/*!< in: doc id to lookup */
@@ -442,19 +442,6 @@ fts_cache_append_deleted_doc_ids(
 			cache,		/*!< in: cache to use */
 	ib_vector_t*	vector);	/*!< in: append to this vector */
 /******************************************************************//**
-Wait for the background thread to start. We poll to detect change
-of state, which is acceptable, since the wait should happen only
-once during startup.
-@return true if the thread started else FALSE (i.e timed out) */
-ibool
-fts_wait_for_background_thread_to_start(
-/*====================================*/
-	dict_table_t*	table,		/*!< in: table to which the thread
-					is attached */
-	ulint		max_wait);	/*!< in: time in microseconds, if set
-					to 0 then it disables timeout
-					checking */
-/******************************************************************//**
 Search the index specific cache for a particular FTS index.
 @return the index specific cache else NULL */
 fts_index_cache_t*
@@ -474,11 +461,7 @@ int
 fts_write_object_id(
 /*================*/
 	ib_id_t		id,		/*!< in: a table/index id */
-	char*		str,		/*!< in: buffer to write the id to */
-	bool		hex_format MY_ATTRIBUTE((unused)))
-					/*!< in: true for fixed hex format,
-					false for old ambiguous format */
-	MY_ATTRIBUTE((nonnull));
+	char*		str);		/*!< in: buffer to write the id to */
 /******************************************************************//**
 Read the table id from the string generated by fts_write_object_id().
 @return TRUE if parse successful */
@@ -503,7 +486,7 @@ fts_get_table_id(
 	MY_ATTRIBUTE((nonnull, warn_unused_result));
 /** Construct the name of an internal FTS table for the given table.
 @param[in]	fts_table	metadata on fulltext-indexed table
-@param[in]	dict_locked	whether dict_sys->mutex is being held
+@param[in]	dict_locked	whether dict_sys.mutex is being held
 @return	the prefix, must be freed with ut_free() */
 char* fts_get_table_name_prefix(const fts_table_t* fts_table)
 	MY_ATTRIBUTE((nonnull, malloc, warn_unused_result));
