@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (C) 2013, 2015, Google Inc. All Rights Reserved.
-Copyright (C) 2014, 2018, MariaDB Corporation.
+Copyright (C) 2014, 2021, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -104,6 +104,7 @@ static bool init_crypt_key(crypt_info_t* info, bool upgrade = false)
 			<< info->key_version << " failed (" << rc
 			<< "). Maybe the key or the required encryption "
 			"key management plugin was not found.";
+		info->key_version = ENCRYPTION_KEY_VERSION_INVALID;
 		return false;
 	}
 
@@ -123,6 +124,7 @@ static bool init_crypt_key(crypt_info_t* info, bool upgrade = false)
 	if (err != MY_AES_OK || dst_len != MY_AES_BLOCK_SIZE) {
 		ib::error() << "Getting redo log crypto key failed: err = "
 			<< err << ", len = " << dst_len;
+		info->key_version = ENCRYPTION_KEY_VERSION_INVALID;
 		return false;
 	}
 
@@ -173,10 +175,10 @@ bool log_crypt(byte* buf, lsn_t lsn, ulint size, log_crypt_t op)
 		byte* key_ver = &buf[OS_FILE_LOG_BLOCK_SIZE - LOG_BLOCK_KEY
 				     - LOG_BLOCK_CHECKSUM];
 		const uint dst_size
-			= log_sys.log.format == LOG_HEADER_FORMAT_ENC_10_4
+			= log_sys.log.format == log_t::FORMAT_ENC_10_4
 			? sizeof dst - LOG_BLOCK_KEY
 			: sizeof dst;
-		if (log_sys.log.format == LOG_HEADER_FORMAT_ENC_10_4) {
+		if (log_sys.log.format == log_t::FORMAT_ENC_10_4) {
 			const uint key_version = info.key_version;
 			switch (op) {
 			case LOG_ENCRYPT_ROTATE_KEY:
@@ -317,6 +319,7 @@ log_crypt_101_read_block(byte* buf)
 	for (const crypt_info_t* const end = info + infos_used; info < end;
 	     info++) {
 		if (info->key_version
+		    && info->key_version != ENCRYPTION_KEY_VERSION_INVALID
 		    && info->checkpoint_no == checkpoint_no) {
 			goto found;
 		}
@@ -328,6 +331,9 @@ log_crypt_101_read_block(byte* buf)
 	/* MariaDB Server 10.1 would use the first key if it fails to
 	find a key for the current checkpoint. */
 	info = infos;
+	if (info->key_version == ENCRYPTION_KEY_VERSION_INVALID) {
+		return false;
+	}
 found:
 	byte dst[OS_FILE_LOG_BLOCK_SIZE];
 	uint dst_len;
