@@ -52,6 +52,11 @@ Created 12/27/1996 Heikki Tuuri
 #include <algorithm>
 #include <mysql/plugin.h>
 #include <mysql/service_wsrep.h>
+#ifdef WITH_WSREP
+#include "log.h"
+#include "wsrep.h"
+#endif /* WITH_WSREP */
+
 
 /* What kind of latch and lock can we assume when the control comes to
    -------------------------------------------------------------------
@@ -126,25 +131,23 @@ NOTE that since we do not hold dict_sys.latch when leaving the
 function, it may be that the referencing table has been dropped when
 we leave this function: this function is only for heuristic use!
 
-@return TRUE if referenced */
+@return true if referenced */
 static
-ibool
+bool
 row_upd_index_is_referenced(
 /*========================*/
 	dict_index_t*	index,	/*!< in: index */
 	trx_t*		trx)	/*!< in: transaction */
 {
 	dict_table_t*	table		= index->table;
-	ibool		froze_data_dict	= FALSE;
-	ibool		is_referenced	= FALSE;
 
 	if (table->referenced_set.empty()) {
-		return(FALSE);
+		return false;
 	}
 
-	if (trx->dict_operation_lock_mode == 0) {
+	const bool froze_data_dict = !trx->dict_operation_lock_mode;
+	if (froze_data_dict) {
 		row_mysql_freeze_data_dictionary(trx);
-		froze_data_dict = TRUE;
 	}
 
 	dict_foreign_set::iterator	it
@@ -152,13 +155,13 @@ row_upd_index_is_referenced(
 			       table->referenced_set.end(),
 			       dict_foreign_with_index(index));
 
-	is_referenced = (it != table->referenced_set.end());
+	const bool is_referenced = (it != table->referenced_set.end());
 
 	if (froze_data_dict) {
 		row_mysql_unfreeze_data_dictionary(trx);
 	}
 
-	return(is_referenced);
+	return is_referenced;
 }
 
 #ifdef WITH_WSREP
@@ -1002,7 +1005,8 @@ row_upd_build_difference_binary(
 	n_diff = 0;
 
 	if (!offsets) {
-		offsets = rec_get_offsets(rec, index, offsets_, true,
+		offsets = rec_get_offsets(rec, index, offsets_,
+					  index->n_core_fields,
 					  ULINT_UNDEFINED, &heap);
 	} else {
 		ut_ad(rec_offs_validate(rec, index, offsets));
@@ -2203,7 +2207,8 @@ row_upd_store_row(
 
 	rec = btr_pcur_get_rec(node->pcur);
 
-	offsets = rec_get_offsets(rec, clust_index, offsets_, true,
+	offsets = rec_get_offsets(rec, clust_index, offsets_,
+				  clust_index->n_core_fields,
 				  ULINT_UNDEFINED, &heap);
 
 	if (dict_table_has_atomic_blobs(node->table)) {
@@ -2264,7 +2269,6 @@ row_upd_sec_index_entry(
 	dtuple_t*		entry;
 	dict_index_t*		index;
 	btr_cur_t*		btr_cur;
-	ibool			referenced;
 	dberr_t			err	= DB_SUCCESS;
 	trx_t*			trx	= thr_get_trx(thr);
 	ulint			mode;
@@ -2275,7 +2279,7 @@ row_upd_sec_index_entry(
 
 	index = node->index;
 
-	referenced = row_upd_index_is_referenced(index, trx);
+	const bool referenced = row_upd_index_is_referenced(index, trx);
 #ifdef WITH_WSREP
 	bool foreign = wsrep_row_upd_index_is_foreign(index, trx);
 #endif /* WITH_WSREP */
@@ -2437,7 +2441,7 @@ row_upd_sec_index_entry(
 			    && !wsrep_thd_is_BF(trx->mysql_thd, FALSE)) {
 
 				rec_offs* offsets = rec_get_offsets(
-					rec, index, NULL, true,
+					rec, index, NULL, index->n_core_fields,
 					ULINT_UNDEFINED, &heap);
 
 				err = wsrep_row_upd_check_foreign_constraints(
@@ -2450,42 +2454,35 @@ row_upd_sec_index_entry(
 					err = DB_SUCCESS;
 					break;
 				case DB_LOCK_WAIT:
-					if (UNIV_UNLIKELY(wsrep_debug)) {
-						ib::warn() << "WSREP: sec index FK lock wait"
-							   << " index " << index->name
-							   << " table " << index->table->name
-							   << " query " << wsrep_thd_query(trx->mysql_thd);
-					}
-					break;
 				case DB_DEADLOCK:
-					if (UNIV_UNLIKELY(wsrep_debug)) {
-						ib::warn() << "WSREP: sec index FK check fail for deadlock"
-							   << " index " << index->name
-							   << " table " << index->table->name
-							   << " query " << wsrep_thd_query(trx->mysql_thd);
-					}
+				case DB_LOCK_WAIT_TIMEOUT:
+					WSREP_DEBUG("Foreign key check fail: "
+						"%s on table %s index %s query %s",
+						ut_strerr(err), index->name(), index->table->name.m_name,
+						wsrep_thd_query(trx->mysql_thd));
 					break;
 				default:
-					ib::error() << "WSREP: referenced FK check fail: " << err
-						    << " index " << index->name
-						    << " table " << index->table->name
-						    << " query " << wsrep_thd_query(trx->mysql_thd);
-
+					WSREP_ERROR("Foreign key check fail: "
+						"%s on table %s index %s query %s",
+						ut_strerr(err), index->name(), index->table->name.m_name,
+						wsrep_thd_query(trx->mysql_thd));
 					break;
 				}
 			}
 #endif /* WITH_WSREP */
 		}
 
+#ifdef WITH_WSREP
+		ut_ad(err == DB_SUCCESS || err == DB_LOCK_WAIT
+		      || err == DB_DEADLOCK || err == DB_LOCK_WAIT_TIMEOUT);
+#else
 		ut_ad(err == DB_SUCCESS);
+#endif
 
 		if (referenced) {
-
-			rec_offs* offsets;
-
-			offsets = rec_get_offsets(
-				rec, index, NULL, true, ULINT_UNDEFINED,
-				&heap);
+			rec_offs* offsets = rec_get_offsets(
+				rec, index, NULL, index->n_core_fields,
+				ULINT_UNDEFINED, &heap);
 
 			/* NOTE that the following call loses
 			the position of pcur ! */
@@ -2682,12 +2679,13 @@ row_upd_clust_rec_by_insert(
 	upd_node_t*	node,	/*!< in/out: row update node */
 	dict_index_t*	index,	/*!< in: clustered index of the record */
 	que_thr_t*	thr,	/*!< in: query thread */
-	ibool		referenced,/*!< in: TRUE if index may be referenced in
+	bool		referenced,/*!< in: whether index may be referenced in
 				a foreign key constraint */
 #ifdef WITH_WSREP
 	bool		foreign,/*!< in: whether this is a foreign key */
 #endif
-	mtr_t*		mtr)	/*!< in/out: mtr; gets committed here */
+	mtr_t*		mtr)	/*!< in/out: mini-transaction,
+				may be committed and restarted */
 {
 	mem_heap_t*	heap;
 	btr_pcur_t*	pcur;
@@ -2736,7 +2734,8 @@ row_upd_clust_rec_by_insert(
 		we update the primary key.  Delete-mark the old record
 		in the clustered index and prepare to insert a new entry. */
 		rec = btr_cur_get_rec(btr_cur);
-		offsets = rec_get_offsets(rec, index, offsets, true,
+		offsets = rec_get_offsets(rec, index, offsets,
+					  index->n_core_fields,
 					  ULINT_UNDEFINED, &heap);
 		ut_ad(page_rec_is_user_rec(rec));
 
@@ -2757,10 +2756,7 @@ row_upd_clust_rec_by_insert(
 			btr_cur_get_block(btr_cur), rec, index, offsets,
 			thr, node->row, mtr);
 		if (err != DB_SUCCESS) {
-err_exit:
-			mtr_commit(mtr);
-			mem_heap_free(heap);
-			return(err);
+			goto err_exit;
 		}
 
 		/* If the the new row inherits externally stored
@@ -2803,31 +2799,35 @@ check_fk:
 			case DB_NO_REFERENCED_ROW:
 				err = DB_SUCCESS;
 				break;
+			case DB_LOCK_WAIT:
 			case DB_DEADLOCK:
-				if (UNIV_UNLIKELY(wsrep_debug)) {
-					ib::warn() << "WSREP: sec index FK check fail for deadlock"
-						   << " index " << index->name
-						   << " table " << index->table->name;
-				}
+			case DB_LOCK_WAIT_TIMEOUT:
+				WSREP_DEBUG("Foreign key check fail: "
+					    "%s on table %s index %s query %s",
+					    ut_strerr(err), index->name(), index->table->name.m_name,
+					    wsrep_thd_query(trx->mysql_thd));
+
 				goto err_exit;
 			default:
-				ib::error() << "WSREP: referenced FK check fail: " << err
-					    << " index " << index->name
-					    << " table " << index->table->name;
+				WSREP_ERROR("Foreign key check fail: "
+					    "%s on table %s index %s query %s",
+					    ut_strerr(err), index->name(), index->table->name.m_name,
+					    wsrep_thd_query(trx->mysql_thd));
+
 				goto err_exit;
 			}
 #endif /* WITH_WSREP */
 		}
 	}
 
-	mtr_commit(mtr);
+	mtr->commit();
+	mtr->start();
 
+	node->state = UPD_NODE_INSERT_CLUSTERED;
 	err = row_ins_clust_index_entry(index, entry, thr,
 					dtuple_get_n_ext(entry));
-	node->state = UPD_NODE_INSERT_CLUSTERED;
-
+err_exit:
 	mem_heap_free(heap);
-
 	return(err);
 }
 
@@ -2847,7 +2847,8 @@ row_upd_clust_rec(
 	mem_heap_t**	offsets_heap,
 				/*!< in/out: memory heap, can be emptied */
 	que_thr_t*	thr,	/*!< in: query thread */
-	mtr_t*		mtr)	/*!< in: mtr; gets committed here */
+	mtr_t*		mtr)	/*!< in,out: mini-transaction; may be
+				committed and restarted here */
 {
 	mem_heap_t*	heap		= NULL;
 	big_rec_t*	big_rec		= NULL;
@@ -2893,16 +2894,15 @@ row_upd_clust_rec(
 		goto success;
 	}
 
-	mtr_commit(mtr);
-
 	if (buf_LRU_buf_pool_running_out()) {
-
 		err = DB_LOCK_TABLE_FULL;
 		goto func_exit;
 	}
+
 	/* We may have to modify the tree structure: do a pessimistic descent
 	down the index tree */
 
+	mtr->commit();
 	mtr->start();
 
 	if (index->table->is_temporary()) {
@@ -2952,7 +2952,6 @@ success:
 		}
 	}
 
-	mtr_commit(mtr);
 func_exit:
 	if (heap) {
 		mem_heap_free(heap);
@@ -2977,17 +2976,17 @@ row_upd_del_mark_clust_rec(
 	rec_offs*	offsets,/*!< in/out: rec_get_offsets() for the
 				record under the cursor */
 	que_thr_t*	thr,	/*!< in: query thread */
-	ibool		referenced,
-				/*!< in: TRUE if index may be referenced in
+	bool		referenced,
+				/*!< in: whether index may be referenced in
 				a foreign key constraint */
 #ifdef WITH_WSREP
 	bool		foreign,/*!< in: whether this is a foreign key */
 #endif
-	mtr_t*		mtr)	/*!< in: mtr; gets committed here */
+	mtr_t*		mtr)	/*!< in,out: mini-transaction;
+				will be committed and restarted */
 {
 	btr_pcur_t*	pcur;
 	btr_cur_t*	btr_cur;
-	dberr_t		err;
 	rec_t*		rec;
 	trx_t*		trx = thr_get_trx(thr);
 
@@ -3003,8 +3002,7 @@ row_upd_del_mark_clust_rec(
 	if (!row_upd_store_row(node, trx->mysql_thd,
 			  thr->prebuilt  && thr->prebuilt->table == node->table
 			  ? thr->prebuilt->m_mysql_table : NULL)) {
-		err = DB_COMPUTE_VALUE_FAILED;
-		return err;
+		return DB_COMPUTE_VALUE_FAILED;
 	}
 
 	/* Mark the clustered index record deleted; we do not have to check
@@ -3012,7 +3010,7 @@ row_upd_del_mark_clust_rec(
 
 	rec = btr_cur_get_rec(btr_cur);
 
-	err = btr_cur_del_mark_set_clust_rec(
+	dberr_t err = btr_cur_del_mark_set_clust_rec(
 		btr_cur_get_block(btr_cur), rec,
 		index, offsets, thr, node->row, mtr);
 
@@ -3032,25 +3030,24 @@ row_upd_del_mark_clust_rec(
 		case DB_NO_REFERENCED_ROW:
 			err = DB_SUCCESS;
 			break;
+		case DB_LOCK_WAIT:
 		case DB_DEADLOCK:
-			if (UNIV_UNLIKELY(wsrep_debug)) {
-				ib::warn() << "WSREP: sec index FK check fail for deadlock"
-					   << " index " << index->name
-					   << " table " << index->table->name;
-			}
+		case DB_LOCK_WAIT_TIMEOUT:
+			WSREP_DEBUG("Foreign key check fail: "
+				    "%d on table %s index %s query %s",
+				    err, index->name(), index->table->name.m_name,
+				    wsrep_thd_query(trx->mysql_thd));
 			break;
 		default:
-			ib::error() << "WSREP: referenced FK check fail: " << err
-				    << " index " << index->name
-				    << " table " << index->table->name;
-
+			WSREP_ERROR("Foreign key check fail: "
+				    "%d on table %s index %s query %s",
+				    err, index->name(), index->table->name.m_name,
+				    wsrep_thd_query(trx->mysql_thd));
 			break;
 		}
 #endif /* WITH_WSREP */
 	}
 
-	mtr_commit(mtr);
-
 	return(err);
 }
 
@@ -3067,14 +3064,12 @@ row_upd_clust_step(
 {
 	dict_index_t*	index;
 	btr_pcur_t*	pcur;
-	ibool		success;
 	dberr_t		err;
 	mtr_t		mtr;
 	rec_t*		rec;
 	mem_heap_t*	heap	= NULL;
 	rec_offs	offsets_[REC_OFFS_NORMAL_SIZE];
 	rec_offs*	offsets;
-	ibool		referenced;
 	ulint		flags;
 	trx_t*		trx = thr_get_trx(thr);
 
@@ -3082,8 +3077,7 @@ row_upd_clust_step(
 
 	index = dict_table_get_first_index(node->table);
 
-	referenced = row_upd_index_is_referenced(index, trx);
-
+	const bool referenced = row_upd_index_is_referenced(index, trx);
 #ifdef WITH_WSREP
 	const bool foreign = wsrep_row_upd_index_is_foreign(index, trx);
 #endif
@@ -3129,14 +3123,9 @@ row_upd_clust_step(
 		mode = BTR_MODIFY_LEAF;
 	}
 
-	success = btr_pcur_restore_position(mode, pcur, &mtr);
-
-	if (!success) {
+	if (!btr_pcur_restore_position(mode, pcur, &mtr)) {
 		err = DB_RECORD_NOT_FOUND;
-
-		mtr_commit(&mtr);
-
-		return(err);
+		goto exit_func;
 	}
 
 	/* If this is a row in SYS_INDEXES table of the data dictionary,
@@ -3156,19 +3145,14 @@ row_upd_clust_step(
 		mtr.start();
 		index->set_modified(mtr);
 
-		success = btr_pcur_restore_position(BTR_MODIFY_LEAF, pcur,
-						    &mtr);
-		if (!success) {
+		if (!btr_pcur_restore_position(BTR_MODIFY_LEAF, pcur, &mtr)) {
 			err = DB_ERROR;
-
-			mtr.commit();
-
-			return(err);
+			goto exit_func;
 		}
 	}
 
 	rec = btr_pcur_get_rec(pcur);
-	offsets = rec_get_offsets(rec, index, offsets_, true,
+	offsets = rec_get_offsets(rec, index, offsets_, index->n_core_fields,
 				  ULINT_UNDEFINED, &heap);
 
 	if (!flags && !node->has_clust_rec_x_lock) {
@@ -3176,7 +3160,6 @@ row_upd_clust_step(
 			0, btr_pcur_get_block(pcur),
 			rec, index, offsets, thr);
 		if (err != DB_SUCCESS) {
-			mtr.commit();
 			goto exit_func;
 		}
 	}
@@ -3187,8 +3170,6 @@ row_upd_clust_step(
 					  btr_pcur_get_block(pcur),
 					  page_rec_get_heap_no(rec)));
 
-	/* NOTE: the following function calls will also commit mtr */
-
 	if (node->is_delete == PLAIN_DELETE) {
 		err = row_upd_del_mark_clust_rec(
 			node, index, offsets, thr, referenced,
@@ -3196,13 +3177,7 @@ row_upd_clust_step(
 			foreign,
 #endif
 			&mtr);
-
-		if (err == DB_SUCCESS) {
-			node->state = UPD_NODE_UPDATE_ALL_SEC;
-			node->index = dict_table_get_next_index(index);
-		}
-
-		goto exit_func;
+		goto all_done;
 	}
 
 	/* If the update is made for MySQL, we already have the update vector
@@ -3217,14 +3192,13 @@ row_upd_clust_step(
 	}
 
 	if (!node->is_delete && node->cmpl_info & UPD_NODE_NO_ORD_CHANGE) {
-
 		err = row_upd_clust_rec(
 			flags, node, index, offsets, &heap, thr, &mtr);
 		goto exit_func;
 	}
 
-	if(!row_upd_store_row(node, trx->mysql_thd,
-			thr->prebuilt ? thr->prebuilt->m_mysql_table : NULL)) {
+	if (!row_upd_store_row(node, trx->mysql_thd, thr->prebuilt
+			       ? thr->prebuilt->m_mysql_table : NULL)) {
 		err = DB_COMPUTE_VALUE_FAILED;
 		goto exit_func;
 	}
@@ -3249,34 +3223,31 @@ row_upd_clust_step(
 			foreign,
 #endif
 			&mtr);
-		if (err != DB_SUCCESS) {
-
-			goto exit_func;
+all_done:
+		if (err == DB_SUCCESS) {
+			node->state = UPD_NODE_UPDATE_ALL_SEC;
+success:
+			node->index = dict_table_get_next_index(index);
 		}
-
-		node->state = UPD_NODE_UPDATE_ALL_SEC;
 	} else {
 		err = row_upd_clust_rec(
 			flags, node, index, offsets, &heap, thr, &mtr);
 
-		if (err != DB_SUCCESS) {
-
-			goto exit_func;
+		if (err == DB_SUCCESS) {
+			ut_ad(node->is_delete != PLAIN_DELETE);
+			node->state = node->is_delete
+				? UPD_NODE_UPDATE_ALL_SEC
+				: UPD_NODE_UPDATE_SOME_SEC;
+			goto success;
 		}
-
-		ut_ad(node->is_delete != PLAIN_DELETE);
-		node->state = node->is_delete ?
-			UPD_NODE_UPDATE_ALL_SEC :
-			UPD_NODE_UPDATE_SOME_SEC;
 	}
 
-	node->index = dict_table_get_next_index(index);
-
 exit_func:
-	if (heap) {
+	mtr.commit();
+	if (UNIV_LIKELY_NULL(heap)) {
 		mem_heap_free(heap);
 	}
-	return(err);
+	return err;
 }
 
 /***********************************************************//**
