@@ -39,7 +39,7 @@ static void maria_scan_end_dummy(MARIA_HA *info);
 static my_bool maria_once_init_dummy(MARIA_SHARE *, File);
 static my_bool maria_once_end_dummy(MARIA_SHARE *);
 static uchar *_ma_base_info_read(uchar *ptr, MARIA_BASE_INFO *base);
-static uchar *_ma_state_info_read(uchar *ptr, MARIA_STATE_INFO *state);
+static uchar *_ma_state_info_read(uchar *, MARIA_STATE_INFO *, myf);
 
 #define get_next_element(to,pos,size) { memcpy((char*) to,pos,(size_t) size); \
 					pos+=size;}
@@ -98,6 +98,7 @@ static MARIA_HA *maria_clone_internal(MARIA_SHARE *share,
   uint errpos;
   MARIA_HA info,*m_info;
   my_bitmap_map *changed_fields_bitmap;
+  myf flag= MY_WME | (share->temporary ? MY_THREAD_SPECIFIC : 0);
   DBUG_ENTER("maria_clone_internal");
 
   errpos= 0;
@@ -115,7 +116,7 @@ static MARIA_HA *maria_clone_internal(MARIA_SHARE *share,
   errpos= 5;
 
   /* alloc and set up private structure parts */
-  if (!my_multi_malloc(MY_WME,
+  if (!my_multi_malloc(flag,
 		       &m_info,sizeof(MARIA_HA),
 		       &info.blobs,sizeof(MARIA_BLOB)*share->base.blobs,
 		       &info.buff,(share->base.max_key_block_length*2+
@@ -163,10 +164,9 @@ static MARIA_HA *maria_clone_internal(MARIA_SHARE *share,
     goto err;
 
   /* The following should be big enough for all pinning purposes */
-  if (my_init_dynamic_array(&info.pinned_pages,
-                            sizeof(MARIA_PINNED_PAGE),
+  if (my_init_dynamic_array(&info.pinned_pages, sizeof(MARIA_PINNED_PAGE),
                             MY_MAX(share->base.blobs*2 + 4,
-                                MARIA_MAX_TREE_LEVELS*3), 16, MYF(0)))
+                            MARIA_MAX_TREE_LEVELS*3), 16, flag))
     goto err;
 
 
@@ -202,7 +202,7 @@ static MARIA_HA *maria_clone_internal(MARIA_SHARE *share,
   /* Allocate buffer for one record */
   /* prerequisites: info->rec_buffer == 0 && info->rec_buff_size == 0 */
   if (_ma_alloc_buffer(&info.rec_buff, &info.rec_buff_size,
-                       share->base.default_rec_buff_size))
+                       share->base.default_rec_buff_size, flag))
     goto err;
 
   bzero(info.rec_buff, share->base.default_rec_buff_size);
@@ -265,6 +265,7 @@ MARIA_HA *maria_open(const char *name, int mode, uint open_flags)
   uint i,j,len,errpos,head_length,base_pos,keys, realpath_err,
     key_parts,base_key_parts,unique_key_parts,fulltext_keys,uniques;
   uint internal_table= MY_TEST(open_flags & HA_OPEN_INTERNAL_TABLE);
+  myf common_flag= open_flags & HA_OPEN_TMP_TABLE ? MY_THREAD_SPECIFIC : 0;
   uint file_version;
   size_t info_length;
   char name_buff[FN_REFLEN], org_name[FN_REFLEN], index_name[FN_REFLEN],
@@ -322,13 +323,13 @@ MARIA_HA *maria_open(const char *name, int mode, uint open_flags)
     DEBUG_SYNC_C("mi_open_kfile");
     if ((kfile=mysql_file_open(key_file_kfile, name_buff,
                                (open_mode=O_RDWR) | O_SHARE | O_NOFOLLOW | O_CLOEXEC,
-                               MYF(MY_NOSYMLINKS))) < 0)
+                               MYF(common_flag | MY_NOSYMLINKS))) < 0)
     {
       if ((errno != EROFS && errno != EACCES) ||
 	  mode != O_RDONLY ||
 	  (kfile=mysql_file_open(key_file_kfile, name_buff,
                                  (open_mode=O_RDONLY) | O_SHARE | O_NOFOLLOW | O_CLOEXEC,
-                                 MYF(MY_NOSYMLINKS))) < 0)
+                                 MYF(common_flag | MY_NOSYMLINKS))) < 0)
 	goto err;
     }
     share->mode=open_mode;
@@ -393,7 +394,7 @@ MARIA_HA *maria_open(const char *name, int mode, uint open_flags)
       Allocate space for header information and for data that is too
       big to keep on stack
     */
-    if (!(disk_cache= my_malloc(info_length+128, MYF(MY_WME))))
+    if (!(disk_cache= my_malloc(info_length+128, MYF(MY_WME | common_flag))))
     {
       my_errno=ENOMEM;
       goto err;
@@ -420,7 +421,7 @@ MARIA_HA *maria_open(const char *name, int mode, uint open_flags)
     }
     share->state_diff_length=len-MARIA_STATE_INFO_SIZE;
 
-    if (!_ma_state_info_read(disk_cache, &share->state))
+    if (!_ma_state_info_read(disk_cache, &share->state, common_flag))
       goto err;
     len= mi_uint2korr(share->state.header.base_info_length);
     if (len != MARIA_BASE_INFO_SIZE)
@@ -561,12 +562,10 @@ MARIA_HA *maria_open(const char *name, int mode, uint open_flags)
     share->index_file_name.length=  strlen(index_name);
     share->data_file_name.length=   strlen(data_name);
     share->open_file_name.length=   strlen(name);
-    if (!my_multi_malloc(MY_WME,
+    if (!my_multi_malloc(MYF(MY_WME | common_flag),
 			 &share,sizeof(*share),
-			 &rec_per_key_part,
-                         sizeof(double) * key_parts,
-                         &nulls_per_key_part,
-                         sizeof(long)* key_parts,
+			 &rec_per_key_part, sizeof(double) * key_parts,
+                         &nulls_per_key_part, sizeof(long)* key_parts,
 			 &share->keyinfo,keys*sizeof(MARIA_KEYDEF),
 			 &share->uniqueinfo,uniques*sizeof(MARIA_UNIQUEDEF),
 			 &share->keyparts,
@@ -883,9 +882,9 @@ MARIA_HA *maria_open(const char *name, int mode, uint open_flags)
       share->options|= HA_OPTION_READ_ONLY_DATA;
     share->is_log_table= FALSE;
 
-    if (open_flags & HA_OPEN_TMP_TABLE ||
-        (share->options & HA_OPTION_TMP_TABLE))
+    if (open_flags & HA_OPEN_TMP_TABLE || share->options & HA_OPTION_TMP_TABLE)
     {
+      common_flag|= MY_THREAD_SPECIFIC;
       share->options|= HA_OPTION_TMP_TABLE;
       share->temporary= share->delay_key_write= 1;
       share->write_flag=MYF(MY_NABP);
@@ -1113,13 +1112,13 @@ err:
 */
 
 my_bool _ma_alloc_buffer(uchar **old_addr, size_t *old_size,
-                         size_t new_size)
+                         size_t new_size, myf flag)
 {
   if (*old_size < new_size)
   {
     uchar *addr;
     if (!(addr= (uchar*) my_realloc(*old_addr, new_size,
-                                    MYF(MY_ALLOW_ZERO_PTR))))
+                                    MYF(MY_ALLOW_ZERO_PTR | flag))))
       return 1;
     *old_addr= addr;
     *old_size= new_size;
@@ -1498,7 +1497,7 @@ uint _ma_state_info_write_sub(File file, MARIA_STATE_INFO *state, uint pWrite)
 }
 
 
-static uchar *_ma_state_info_read(uchar *ptr, MARIA_STATE_INFO *state)
+static uchar *_ma_state_info_read(uchar *ptr, MARIA_STATE_INFO *state, myf flag)
 {
   uint i,keys,key_parts;
   DBUG_ENTER("_ma_state_info_read");
@@ -1510,7 +1509,7 @@ static uchar *_ma_state_info_read(uchar *ptr, MARIA_STATE_INFO *state)
 
   /* Allocate memory for key parts if not already done */
   if (!state->rec_per_key_part &&
-      !my_multi_malloc(MY_WME,
+      !my_multi_malloc(MYF(MY_WME | flag),
                        &state->rec_per_key_part,
                        sizeof(*state->rec_per_key_part) * key_parts,
                        &state->nulls_per_key_part,
@@ -1955,6 +1954,8 @@ void _ma_set_index_pagecache_callbacks(PAGECACHE_FILE *file,
 int _ma_open_datafile(MARIA_HA *info, MARIA_SHARE *share)
 {
   myf flags= MY_WME | (share->mode & O_NOFOLLOW ? MY_NOSYMLINKS : 0);
+  if (share->temporary)
+    flags|= MY_THREAD_SPECIFIC;
   DEBUG_SYNC_C("mi_open_datafile");
   info->dfile.file= share->bitmap.file.file=
     mysql_file_open(key_file_dfile, share->data_file_name.str,
