@@ -1198,7 +1198,7 @@ extern my_bool _ma_read_cache(MARIA_HA *, IO_CACHE *info, uchar *buff,
                               uint re_read_if_possibly);
 extern ulonglong ma_retrieve_auto_increment(const uchar *key, uint8 key_type);
 extern my_bool _ma_alloc_buffer(uchar **old_addr, size_t *old_size,
-                                size_t new_size);
+                                size_t new_size, myf flag);
 extern size_t _ma_rec_unpack(MARIA_HA *info, uchar *to, uchar *from,
                             size_t reclength);
 extern my_bool _ma_rec_check(MARIA_HA *info, const uchar *record,
