@@ -49,45 +49,45 @@ table_tiws_by_index_usage::m_share=
   sizeof(pos_tiws_by_index_usage),
   &m_table_lock,
   { C_STRING_WITH_LEN("CREATE TABLE table_io_waits_summary_by_index_usage("
-                      "OBJECT_TYPE VARCHAR(64) comment 'TABLE in the case of all indexes.',"
-                      "OBJECT_SCHEMA VARCHAR(64) comment 'Schema name.',"
-                      "OBJECT_NAME VARCHAR(64) comment 'Table name.',"
-                      "INDEX_NAME VARCHAR(64) comment 'Index name, or PRIMARY for the primary index, NULL for no index (inserts are counted in this case).',"
-                      "COUNT_STAR BIGINT unsigned not null comment 'Number of summarized events and the sum of the x_READ and x_WRITE columns.',"
-                      "SUM_TIMER_WAIT BIGINT unsigned not null comment 'Total wait time of the summarized events that are timed.',"
-                      "MIN_TIMER_WAIT BIGINT unsigned not null comment 'Minimum wait time of the summarized events that are timed.',"
-                      "AVG_TIMER_WAIT BIGINT unsigned not null comment 'Average wait time of the summarized events that are timed.',"
-                      "MAX_TIMER_WAIT BIGINT unsigned not null comment 'Maximum wait time of the summarized events that are timed.',"
-                      "COUNT_READ BIGINT unsigned not null comment 'Number of all read operations, and the sum of the equivalent x_FETCH columns.',"
-                      "SUM_TIMER_READ BIGINT unsigned not null comment 'Total wait time of all read operations that are timed.',"
-                      "MIN_TIMER_READ BIGINT unsigned not null comment 'Minimum wait time of all read operations that are timed.',"
-                      "AVG_TIMER_READ BIGINT unsigned not null comment 'Average wait time of all read operations that are timed.',"
-                      "MAX_TIMER_READ BIGINT unsigned not null comment 'Maximum wait time of all read operations that are timed.',"
-                      "COUNT_WRITE BIGINT unsigned not null comment 'Number of all write operations, and the sum of the equivalent x_INSERT, x_UPDATE and x_DELETE columns.',"
-                      "SUM_TIMER_WRITE BIGINT unsigned not null comment 'Total wait time of all write operations that are timed.',"
-                      "MIN_TIMER_WRITE BIGINT unsigned not null comment 'Minimum wait time of all write operations that are timed.',"
-                      "AVG_TIMER_WRITE BIGINT unsigned not null comment 'Average wait time of all write operations that are timed.',"
-                      "MAX_TIMER_WRITE BIGINT unsigned not null comment 'Maximum wait time of all write operations that are timed.',"
-                      "COUNT_FETCH BIGINT unsigned not null comment 'Number of all fetch operations.',"
-                      "SUM_TIMER_FETCH BIGINT unsigned not null comment 'Total wait time of all fetch operations that are timed.',"
-                      "MIN_TIMER_FETCH BIGINT unsigned not null comment 'Minimum wait time of all fetch operations that are timed.',"
-                      "AVG_TIMER_FETCH BIGINT unsigned not null comment 'Average wait time of all fetch operations that are timed.',"
-                      "MAX_TIMER_FETCH BIGINT unsigned not null comment 'Maximum wait time of all fetch operations that are timed.',"
-                      "COUNT_INSERT BIGINT unsigned not null comment 'Number of all insert operations.',"
-                      "SUM_TIMER_INSERT BIGINT unsigned not null comment 'Total wait time of all insert operations that are timed.',"
-                      "MIN_TIMER_INSERT BIGINT unsigned not null comment 'Minimum wait time of all insert operations that are timed.',"
-                      "AVG_TIMER_INSERT BIGINT unsigned not null comment 'Average wait time of all insert operations that are timed.',"
-                      "MAX_TIMER_INSERT BIGINT unsigned not null comment 'Maximum wait time of all insert operations that are timed.',"
-                      "COUNT_UPDATE BIGINT unsigned not null comment 'Number of all update operations.',"
-                      "SUM_TIMER_UPDATE BIGINT unsigned not null comment 'Total wait time of all update operations that are timed.',"
-                      "MIN_TIMER_UPDATE BIGINT unsigned not null comment 'Minimum wait time of all update operations that are timed.',"
-                      "AVG_TIMER_UPDATE BIGINT unsigned not null comment 'Average wait time of all update operations that are timed.',"
-                      "MAX_TIMER_UPDATE BIGINT unsigned not null comment 'Maximum wait time of all update operations that are timed.',"
-                      "COUNT_DELETE BIGINT unsigned not null comment 'Number of all delete operations.',"
-                      "SUM_TIMER_DELETE BIGINT unsigned not null comment 'Total wait time of all delete operations that are timed.',"
-                      "MIN_TIMER_DELETE BIGINT unsigned not null comment 'Minimum wait time of all delete operations that are timed.',"
-                      "AVG_TIMER_DELETE BIGINT unsigned not null comment 'Average wait time of all delete operations that are timed.',"
-                      "MAX_TIMER_DELETE BIGINT unsigned not null comment 'Maximum wait time of all delete operations that are timed.')") }
+                      "OBJECT_TYPE VARCHAR(64),"
+                      "OBJECT_SCHEMA VARCHAR(64),"
+                      "OBJECT_NAME VARCHAR(64),"
+                      "INDEX_NAME VARCHAR(64),"
+                      "COUNT_STAR BIGINT unsigned not null,"
+                      "SUM_TIMER_WAIT BIGINT unsigned not null,"
+                      "MIN_TIMER_WAIT BIGINT unsigned not null,"
+                      "AVG_TIMER_WAIT BIGINT unsigned not null,"
+                      "MAX_TIMER_WAIT BIGINT unsigned not null,"
+                      "COUNT_READ BIGINT unsigned not null,"
+                      "SUM_TIMER_READ BIGINT unsigned not null,"
+                      "MIN_TIMER_READ BIGINT unsigned not null,"
+                      "AVG_TIMER_READ BIGINT unsigned not null,"
+                      "MAX_TIMER_READ BIGINT unsigned not null,"
+                      "COUNT_WRITE BIGINT unsigned not null,"
+                      "SUM_TIMER_WRITE BIGINT unsigned not null,"
+                      "MIN_TIMER_WRITE BIGINT unsigned not null,"
+                      "AVG_TIMER_WRITE BIGINT unsigned not null,"
+                      "MAX_TIMER_WRITE BIGINT unsigned not null,"
+                      "COUNT_FETCH BIGINT unsigned not null,"
+                      "SUM_TIMER_FETCH BIGINT unsigned not null,"
+                      "MIN_TIMER_FETCH BIGINT unsigned not null,"
+                      "AVG_TIMER_FETCH BIGINT unsigned not null,"
+                      "MAX_TIMER_FETCH BIGINT unsigned not null,"
+                      "COUNT_INSERT BIGINT unsigned not null,"
+                      "SUM_TIMER_INSERT BIGINT unsigned not null,"
+                      "MIN_TIMER_INSERT BIGINT unsigned not null,"
+                      "AVG_TIMER_INSERT BIGINT unsigned not null,"
+                      "MAX_TIMER_INSERT BIGINT unsigned not null,"
+                      "COUNT_UPDATE BIGINT unsigned not null,"
+                      "SUM_TIMER_UPDATE BIGINT unsigned not null,"
+                      "MIN_TIMER_UPDATE BIGINT unsigned not null,"
+                      "AVG_TIMER_UPDATE BIGINT unsigned not null,"
+                      "MAX_TIMER_UPDATE BIGINT unsigned not null,"
+                      "COUNT_DELETE BIGINT unsigned not null,"
+                      "SUM_TIMER_DELETE BIGINT unsigned not null,"
+                      "MIN_TIMER_DELETE BIGINT unsigned not null,"
+                      "AVG_TIMER_DELETE BIGINT unsigned not null,"
+                      "MAX_TIMER_DELETE BIGINT unsigned not null)") }
 };
 
 PFS_engine_table*
