@@ -26,6 +26,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 
 #include "hatoku_hton.h"
 #include "PerconaFT/src/ydb.h"
+#include <mutex>
 
 #define TOKU_METADB_NAME "tokudb_meta"
 
@@ -239,6 +240,7 @@ tokudb::thread::rwlock_t tokudb_hton_initialized_lock;
 static SHOW_VAR *toku_global_status_variables = NULL;
 static uint64_t toku_global_status_max_rows;
 static TOKU_ENGINE_STATUS_ROW_S* toku_global_status_rows = NULL;
+static std::mutex toku_global_status_rows_mutex;
 
 static void handle_ydb_error(int error) {
     switch (error) {
@@ -1894,9 +1896,7 @@ static void tokudb_lock_wait_needed_callback(
 // and prefixed with "TOKUDB_"
 static int show_tokudb_vars(TOKUDB_UNUSED(THD* thd),
                             SHOW_VAR* var,
-                            TOKUDB_UNUSED(void* buff),
-                            TOKUDB_UNUSED(system_status_var *status_var),
-                            TOKUDB_UNUSED(enum_var_type var_type)) {
+                            TOKUDB_UNUSED(char* buff)) {
     TOKUDB_DBUG_ENTER("");
 
     int error;
@@ -1904,6 +1904,7 @@ static int show_tokudb_vars(TOKUDB_UNUSED(THD* thd),
     const int panic_string_len = 1024;
     char panic_string[panic_string_len] = {'\0'};
     fs_redzone_state redzone_state;
+    const std::lock_guard<std::mutex> lock(toku_global_status_rows_mutex);
 
     uint64_t num_rows;
     error = db_env->get_engine_status(
@@ -1996,7 +1997,7 @@ static int show_tokudb_vars(TOKUDB_UNUSED(THD* thd),
 }
 
 static SHOW_VAR toku_global_status_variables_export[]= {
-    SHOW_FUNC_ENTRY("Tokudb", &show_tokudb_vars),
+    {"Tokudb", (char*)&show_tokudb_vars, SHOW_FUNC},
     {NullS, NullS, SHOW_LONG}
 };
 
