@@ -401,10 +401,10 @@ my_copy_fix_mb(CHARSET_INFO *cs,
   size_t well_formed_nchars;
   size_t well_formed_length;
   size_t fixed_length;
-  size_t min_length= MY_MIN(src_length, dst_length);
 
+  set_if_smaller(src_length, dst_length);
   well_formed_nchars= cs->cset->well_formed_char_length(cs,
-                                                        src, src + min_length,
+                                                        src, src + src_length,
                                                         nchars, status);
   DBUG_ASSERT(well_formed_nchars <= nchars);
   well_formed_length= status->m_source_end_pos - src;
